﻿using QuestTools;
using QuestTools.Helpers;
using QuestTools.ProfileTags.Complex;
using QuestTools.ProfileTags.Movement;
using System;
using System.Collections.Generic;
using System.Linq;
using Zeta.Bot;
using Zeta.Bot.Logic;
using Zeta.Bot.Profile;
using Zeta.Bot.Profile.Common;
using Zeta.Game;
using Zeta.Game.Internals;
using Zeta.Game.Internals.Actors;
using Zeta.TreeSharp;
using Action = Zeta.TreeSharp.Action;
using Logger = QuestTools.Logger;

namespace KadalaSpree
{
    public class KadalaPlugin : QuestToolsPlugin
    {
        #region Fields

        private static DateTime _lastGambleTime = DateTime.MinValue;
        private DateTime _lastPulseTime = DateTime.MinValue;
        private static List<VendorSlot> _gambleRotation = new List<VendorSlot>();
        private static readonly Random Rnd = new Random();

        #endregion

        private KadalaPlugin()
        {
            Name = "Kadala";
            Version = new Version(2, 7, 2);
            Author = "xzjv";
            Description = "Buys stuff from kadala";
            SettingsClass = KadalaSpreeSettings.Instance;
            SettingsXaml = "Kadala.Settings.xaml";
        }

        public override void OnPulse()
        {
            if (DateTime.UtcNow.Subtract(_lastPulseTime).TotalMilliseconds < 1000)
                return;

            if (!Player.IsValid || !BotBehaviorQueue.IsEnabled)
                return;

            _lastPulseTime = DateTime.UtcNow;

            if (!ShouldGamble) 
                return;

            BotBehaviorQueue.Queue(GambleBehavior, "KadalaSpree");
        }

        /// <summary>
        /// Main gambling sequence.
        /// </summary>
        public List<ProfileBehavior> GambleBehavior = new List<ProfileBehavior>
        {       
            new LogMessageTag
            {
                Output = "Started Gambling Sequence"
            },
            new EnhancedIfTag(
                ret => ZetaDia.IsInTown && !ProfileUtils.CanPathToLocation(ProfileUtils.TownApproachVector),   
                UseWaypointToAct1()
            ),
            new SafeMoveToTag
            {
                SetPositionOnStart = ret => ProfileUtils.TownApproachVector,
                Timeout = 10
            },
            new MoveToActorTag
            {
                ActorId = (int)SNOActor.X1_RandomItemNPC,
                Timeout = 10
            },
            new ActionTag
            {
                IsDoneWhen = ret => !ShouldBuyItems,
                Action = ret => BuyItems()
            },
            new ActionTag(ret =>
            {
                GameUI.CloseVendorWindow();
                return true;
            }),  
            new LogMessageTag
            {
                Output = "Finished Gambling!"
            },
        };

        /// <summary>
        /// Use the waypoint map to travel to Act1
        /// </summary>
        /// <returns></returns>
        public static ProfileBehavior[] UseWaypointToAct1()
        {
            return new ProfileBehavior[]
            {
                new UseWaypointTag
                {
                    QuestId = 1,
                    WaypointNumber = 0
                },
                new WaitTimerTag
                {
                    QuestId = 1,
                    WaitTime = 4000
                }
            };
        }

        /// <summary>
        /// Actually buying items while at the vendor window.
        /// </summary>
        private static bool BuyItems()
        {
            if (!PurchaseDelayPassed)
                return false;

            if (!_gambleRotation.Any())
                _gambleRotation = KadalaSpreeSettings.Instance.SelectedGambleSlots;

            var slot = _gambleRotation[Rnd.Next(_gambleRotation.Count)];
            var itemId = Vendor.MysterySlotTypeAndId[slot];
            var item = ZetaDia.Actors.GetActorsOfType<ACDItem>(true).FirstOrDefault(a => a.ActorSNO == itemId);

            if (item == null)
            {
                Logger.Warn("DB Error ACDItem == null Slot={0} Now buying random item to spend shards", slot);
                var randomItem = ZetaDia.Actors.GetActorsOfType<ACDItem>().FirstOrDefault(a => a.InternalName.StartsWith("PH_"));
                if (randomItem == null)
                    return true;
                
                item = randomItem;
            }
                          
            _gambleRotation.Remove(slot);
            ZetaDia.Me.Inventory.BuyItem(item.DynamicId);
            Logger.Log("Buying: {0}", slot);
            _lastGambleTime = DateTime.UtcNow;

            return false;
        }

        /// <summary>
        /// Determine if we should start he larger gambling process.
        /// </summary>
        private static bool ShouldGamble
        {
            get
            {
                if (!ZetaDia.IsInTown)
                {
                    // Emergency Townrun to spend shards
                    if (KadalaSpreeSettings.Instance.ShouldTownRun && ZetaDia.CPlayer.BloodshardCount >= KadalaSpreeSettings.Instance.SaveShardsThreshold)
                    {
                        var townPortalTag = new CompositeTag()
                        {
                            IsDoneWhen = ret => ZetaDia.IsInTown,
                            Composite = CommonBehaviors.CreateUseTownPortal()
                        };
                        BotBehaviorQueue.Queue(townPortalTag, "Bloodshard Spending Threshold");                        
                    }

                    return false;
                }

                if (ZetaDia.WorldType != Act.OpenWorld || CustomConditions.IsCastingOrLoading())
                    return false;

                if (KadalaSpreeSettings.Instance.SelectedGambleSlots.Count <= 0)
                {
                    LogVerbose("Select at least one thing to buy in settings");
                    return false;
                }

                if (KadalaSpreeSettings.Instance.ShouldSaveShards && ZetaDia.CPlayer.BloodshardCount < KadalaSpreeSettings.Instance.SaveShardsThreshold)
                {
                    LogVerbose("Saving up shards!");
                    return false;
                }

                if (ZetaDia.CPlayer.BloodshardCount < KadalaSpreeSettings.Instance.MinimumBloodShards)
                {
                    LogVerbose("Not enough shards!");
                    return false;
                }

                if (!CanAffordMostExpensiveItem)
                {
                    LogVerbose("Can't afford desired items!");
                    return false;
                }

                if (!QuestTools.Helpers.ItemManager.IsValidTwoSlotLocation)
                {
                    LogVerbose("No Backpack space!");
                    return false;
                }

                LogVerbose("Should Gamble!"); 

                return true;
            }
        }

        /// <summary>
        /// Determine if items can and should be purchased from vendor
        /// </summary>
        private static bool ShouldBuyItems
        {
            get
            {
                if (!ZetaDia.IsInTown || ZetaDia.WorldType != Act.OpenWorld || CustomConditions.IsCastingOrLoading())
                    return false;              

                if (!CustomConditions.IsVendorWindowOpen() || !CustomConditions.ActorExistsNearMe((int)SNOActor.X1_RandomItemNPC,100))
                {
                    LogVerbose("Vendor window is not open or can't find kadala");
                    return false;
                }

                if (!QuestTools.Helpers.ItemManager.IsValidTwoSlotLocation)
                {
                    LogVerbose("No bag space");
                    return false;
                }

                if (ZetaDia.CPlayer.BloodshardCount < KadalaSpreeSettings.Instance.MinimumBloodShards || !CanAffordMostExpensiveItem)
                {
                    LogVerbose("Not enough shards!");
                    return false;
                }

                LogVerbose("Should buy items!");
                return true;
            }
        }

        /// <summary>
        /// Determine if the delay between purchases has passed (attempt to look more human).
        /// </summary>
        public static bool PurchaseDelayPassed
        {
            get
            {
                var timeSinceGamble = DateTime.UtcNow.Subtract(_lastGambleTime).TotalMilliseconds;
                return (_lastGambleTime == DateTime.MinValue || timeSinceGamble > Rnd.Next(50, 350));
            }
        }

        /// <summary>
        /// Determine if there is enough shards for the most expensive of the currently selected items.
        /// </summary>
        public static bool CanAffordMostExpensiveItem
        {
            get
            {
                var slotAndPrice = Vendor.MysterySlotTypeAndPrice.Where(pair => KadalaSpreeSettings.Instance.SelectedGambleSlots.Contains(pair.Key)).ToList();
                return slotAndPrice.Any() && slotAndPrice.Max(pair => pair.Value) <= ZetaDia.CPlayer.BloodshardCount;
            }
        }


        private static void LogVerbose(string msg, params object[] args)
        {
            var debugInfo = String.Format(" Shards={0} SaveShards={1} SaveThreshold={2} CanAffordItem={3} SelectedSlots={4}",
                    ZetaDia.CPlayer.BloodshardCount,
                    KadalaSpreeSettings.Instance.ShouldSaveShards,
                    KadalaSpreeSettings.Instance.SaveShardsThreshold,
                    CanAffordMostExpensiveItem,
                    KadalaSpreeSettings.Instance.SelectedGambleSlots.Count);

            Logger.Verbose(msg + debugInfo, args);
        }

    }

}